@@ -2,7 +2,7 @@
 //
 // proxy.h: Rcpp R/C++ interface class library -- proxies
 //
-// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2014  Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -22,18 +22,19 @@
 #ifndef Rcpp__vector__proxy_h
 #define Rcpp__vector__proxy_h
 
+namespace Rcpp{
 namespace internal{
-	
+
 	template <int RTYPE> class simple_name_proxy {
 	public:
 		typedef ::Rcpp::Vector<RTYPE> VECTOR ;
 		typedef typename ::Rcpp::traits::storage_type<RTYPE>::type CTYPE ;
 		simple_name_proxy( VECTOR& v, const std::string& name_) :
 			parent(v), name(name_){} ;
-		simple_name_proxy( const simple_name_proxy& other ) : 
+		simple_name_proxy( const simple_name_proxy& other ) :
 			parent(other.parent), name(other.name){} ;
 		~simple_name_proxy() {} ;
-		
+
 		simple_name_proxy& operator=( CTYPE rhs ){
 			set( rhs ) ;
 			return *this ;
@@ -42,54 +43,54 @@ namespace internal{
 			set( other.get() ) ;
 			return *this ;
 		}
-		
+
 		template <typename T>
 		simple_name_proxy& operator=( const T& rhs ){
 			set( caster<T,CTYPE>(rhs) ) ;
 			return *this ;
 		}
-		
+
 		// TODO: other operators +=, -=, ...
-		
+
 		operator CTYPE() const {
 			 return get() ;
 		}
-		
+
 		// this helps wrap, for example : wrap( x["foo"] )
 		operator SEXP() const {
 			return ::Rcpp::wrap(get()) ;
 		}
-		
+
 	private:
 		VECTOR& parent ;
 		std::string name;
 		void set( CTYPE rhs ){
-			int index = 0 ;
+			R_xlen_t index = 0 ;
 			try{
 				index = parent.offset(name) ;
 				parent[ index ] = rhs ;
 			} catch( const index_out_of_bounds& ex ){
-				parent.push_back( rhs, name ); 
+				parent.push_back( rhs, name );
 			}
 		}
 		CTYPE get() const {
 			return parent[ parent.offset(name) ];
 		}
 	} ;
-	
+
 	template <int RTYPE>
 	class string_name_proxy{
 	public:
 		typedef typename ::Rcpp::Vector<RTYPE> VECTOR ;
 		typedef const char* iterator ;
 		typedef const char& reference ;
-		
+
 		string_name_proxy( VECTOR& v, const std::string& name_) :
 			parent(v), name(name_){} ;
-		string_name_proxy( const string_name_proxy& other ) : 
+		string_name_proxy( const string_name_proxy& other ) :
 			parent(other.parent), name(other.name){} ;
 		~string_name_proxy(){} ;
-		
+
 		string_name_proxy& operator=( const std::string& rhs ){
 			set( rhs ) ;
 			return *this ;
@@ -98,49 +99,47 @@ namespace internal{
 			set( other.get() ) ;
 			return *this ;
 		}
-		
+
 		operator char* (){
 			 return get() ;
 		}
-		
+
 		operator SEXP(){
 			return ::Rf_mkString(get()) ;
 		}
-		
+
 		inline iterator begin() { return get() ; }
 		inline iterator end(){ return begin() + size() ; }
-		inline reference operator[]( int i ){ return *( get() + i ) ; }
-		inline int size(){ return strlen( get() ) ; }
-		
+		inline reference operator[]( R_xlen_t i ){ return *( get() + i ) ; }
+		inline R_xlen_t size(){ return strlen( get() ) ; }
+
 	private:
 		VECTOR& parent ;
 		std::string name;
 		void set( const std::string& rhs ){
-			int index = 0 ;
+			R_xlen_t index = 0 ;
 			try{
 				index = parent.offset(name) ;
 				parent[ index ] = rhs ;
 			} catch( const index_out_of_bounds& ex ){
-				parent.push_back( rhs, name ); 
+				parent.push_back( rhs, name );
 			}
 		}
 		char* get(){
 			return parent[ parent.offset(name) ];
 		}
-		
+
 	} ;
-	
+
 	template <int RTYPE> class generic_name_proxy {
 	public:
 		typedef ::Rcpp::Vector<RTYPE> VECTOR ;
 		generic_name_proxy( VECTOR& v, const std::string& name_) :
-			parent(v), name(name_){
-				RCPP_DEBUG_2( "generic_name_proxy( VECTOR& = %p, const string& = %s)", v.asSexp(), name_.c_str() );
-		} ;
-		generic_name_proxy( const generic_name_proxy& other ) : 
-			parent(other.parent), name(other.name){} ;
+			parent(v), name(name_){}
+		generic_name_proxy( const generic_name_proxy& other ) :
+			parent(other.parent), name(other.name){}
 		~generic_name_proxy(){} ;
-		
+
 		generic_name_proxy& operator=( SEXP rhs ){
 			set( rhs ) ;
 			return *this ;
@@ -149,19 +148,19 @@ namespace internal{
 			set( other.get() ) ;
 			return *this ;
 		}
-		
+
 		template <typename T>
 		generic_name_proxy& operator=( const T& rhs ){
 			set( ::Rcpp::wrap(rhs) ) ;
 			return *this ;
 		}
-		
+
 		// TODO: other operators +=, -=, ...
-		
+
 		operator SEXP() const {
 			 return get() ;
 		}
-		
+
 		template <typename T>
 		operator T() const {
 			#if RCPP_DEBUG_LEVEL > 0
@@ -172,21 +171,17 @@ namespace internal{
 			return ::Rcpp::as<T>( get() ) ;
 			#endif
 		}
-		
-		operator bool() const{
-		    return ::Rcpp::as<bool>(get()); 
-		}
-		
+
 	private:
 		VECTOR& parent ;
 		std::string name;
 		void set( SEXP rhs ){
-			int index = 0 ;
+			R_xlen_t index = 0 ;
 			try{
 				index = parent.offset(name) ;
 				parent[ index ] = rhs ;
 			} catch( const index_out_of_bounds& ex ){
-				parent.push_back( rhs, name ); 
+				parent.push_back( rhs, name );
 			}
 		}
 		SEXP get() const {
@@ -196,8 +191,8 @@ namespace internal{
 }
 
 namespace traits {
-	
-	template <int RTYPE> 
+
+	template <int RTYPE>
 	struct r_vector_name_proxy{
 		typedef typename ::Rcpp::internal::simple_name_proxy<RTYPE> type ;
 	} ;
@@ -225,19 +220,44 @@ namespace traits {
 		typedef ::Rcpp::internal::generic_proxy<VECSXP> type ;
 	} ;
 
+	template <int RTYPE>
+	struct r_vector_const_proxy{
+		typedef const typename storage_type<RTYPE>::type& type ;
+	} ;
+	template<> struct r_vector_const_proxy<STRSXP> {
+		typedef ::Rcpp::internal::const_string_proxy<STRSXP> type ;
+	} ;
+	template<> struct r_vector_const_proxy<VECSXP> {
+		typedef ::Rcpp::internal::const_generic_proxy<VECSXP> type ;
+	} ;
+	template<> struct r_vector_const_proxy<EXPRSXP> {
+		typedef ::Rcpp::internal::const_generic_proxy<EXPRSXP> type ;
+	} ;
+
 	template <int RTYPE>
 	struct r_vector_iterator {
 		typedef typename storage_type<RTYPE>::type* type ;
 	};
+	template <int RTYPE>
+	struct r_vector_const_iterator {
+		typedef const typename storage_type<RTYPE>::type* type ;
+	};
+
 	template <int RTYPE> struct proxy_based_iterator{
 		typedef ::Rcpp::internal::Proxy_Iterator< typename r_vector_proxy<RTYPE>::type > type ;
 	} ;
 	template<> struct r_vector_iterator<VECSXP> : proxy_based_iterator<VECSXP>{} ;
 	template<> struct r_vector_iterator<EXPRSXP> : proxy_based_iterator<EXPRSXP>{} ;
 	template<> struct r_vector_iterator<STRSXP> : proxy_based_iterator<STRSXP>{} ;
+	
+	template <int RTYPE> struct proxy_based_const_iterator{
+		typedef ::Rcpp::internal::Proxy_Iterator< typename r_vector_const_proxy<RTYPE>::type > type ;
+	} ;
+	template<> struct r_vector_const_iterator<VECSXP> : proxy_based_const_iterator<VECSXP>{} ;
+	template<> struct r_vector_const_iterator<EXPRSXP> : proxy_based_const_iterator<EXPRSXP>{} ;
+	template<> struct r_vector_const_iterator<STRSXP> : proxy_based_const_iterator<STRSXP>{} ;
 
 }  // traits
-
-
+}
 
 #endif
