@@ -2,7 +2,7 @@
 //
 // Vector.cpp: Rcpp R/C++ interface class library -- Vector unit tests
 //
-// Copyright (C) 2012 - 2013    Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2012 - 2015    Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -19,7 +19,10 @@
 // You should have received a copy of the GNU General Public License
 // along with Rcpp.  If not, see <http://www.gnu.org/licenses/>.
 
+#include <climits>
 #include <Rcpp.h>
+#include <sstream>
+
 using namespace Rcpp ;
 
 inline double square( double x){ return x*x; }
@@ -573,6 +576,18 @@ std::string character_iterator1( CharacterVector letters ){
     return res ;
 }
 
+// [[Rcpp::export]]
+std::string character_const_iterator1( const CharacterVector letters ){
+    std::string res ;
+    CharacterVector::const_iterator first = letters.begin() ;
+    CharacterVector::const_iterator last = letters.end() ;
+    while( first != last ){
+        res += *first ;
+        ++first ;
+    }
+    return res ;
+}
+
 
 // [[Rcpp::export]]
 std::string character_iterator2( CharacterVector letters ){
@@ -580,6 +595,12 @@ std::string character_iterator2( CharacterVector letters ){
     return res ;
 }
 
+// [[Rcpp::export]]
+std::string character_const_iterator2( const CharacterVector letters ){
+    std::string res(std::accumulate(letters.begin(), letters.end(), std::string()));
+    return res ;
+}
+
 // [[Rcpp::export]]
 CharacterVector character_reverse( CharacterVector y ){
     std::reverse( y.begin(), y.end() ) ;
@@ -766,3 +787,39 @@ LogicalVector logical_vector_from_bool_assign() {
 void no_op(int major) {
     int minor = 1;
 }
+
+// [[Rcpp::export]]
+int noprotect_vector( Vector<REALSXP, NoProtectStorage> x){
+  return x.size() ;
+}
+
+// [[Rcpp::export]]
+int noprotect_matrix( Matrix<REALSXP, NoProtectStorage> x){
+  return x.nrow() ;
+}
+
+// [[Rcpp::export]]
+int vec_access_with_bounds_checking(const IntegerVector x, int index) {
+    return x.at(index);
+}
+
+// [[Rcpp::export]]
+String vec_print_numeric(NumericVector v) {
+    std::ostringstream buf;
+    buf << v;
+    return buf.str();
+}
+
+// [[Rcpp::export]]
+String vec_print_character(CharacterVector v) {
+    std::ostringstream buf;
+    buf << v;
+    return buf.str();
+}
+
+// [[Rcpp::export]]
+String vec_print_integer(IntegerVector v) {
+    std::ostringstream buf;
+    buf << v;
+    return buf.str();
+}
